@@ -1,8 +1,7 @@
-# -*- coding: utf-8 -*-
-
 from marshmallow import fields
 from marshmallow.utils import is_iterable_but_not_string
 
+from sqlalchemy import inspect
 from sqlalchemy.orm.exc import NoResultFound
 
 
@@ -15,13 +14,6 @@ def get_primary_keys(model):
     return [mapper.get_property_by_column(column) for column in mapper.primary_key]
 
 
-def get_schema_for_field(field):
-    if hasattr(field, "root"):  # marshmallow>=2.1
-        return field.root
-    else:
-        return field.parent
-
-
 def ensure_list(value):
     return value if is_iterable_but_not_string(value) else [value]
 
@@ -50,13 +42,12 @@ class Related(fields.Field):
     }
 
     def __init__(self, column=None, **kwargs):
-        super(Related, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.columns = ensure_list(column or [])
 
     @property
     def model(self):
-        schema = get_schema_for_field(self)
-        return schema.opts.model
+        return self.root.opts.model
 
     @property
     def related_model(self):
@@ -68,20 +59,17 @@ class Related(fields.Field):
     @property
     def related_keys(self):
         if self.columns:
-            return [
-                self.related_model.__mapper__.columns[column] for column in self.columns
-            ]
+            insp = inspect(self.related_model)
+            return [insp.attrs[column] for column in self.columns]
         return get_primary_keys(self.related_model)
 
     @property
     def session(self):
-        schema = get_schema_for_field(self)
-        return schema.session
+        return self.root.session
 
     @property
     def transient(self):
-        schema = get_schema_for_field(self)
-        return schema.transient
+        return self.root.transient
 
     def _serialize(self, value, attr, obj):
         ret = {prop.key: getattr(value, prop.key, None) for prop in self.related_keys}
@@ -96,11 +84,11 @@ class Related(fields.Field):
         """
         if not isinstance(value, dict):
             if len(self.related_keys) != 1:
-                self.fail(
-                    "invalid",
-                    value=value,
-                    keys=[prop.key for prop in self.related_keys],
-                )
+                keys = [prop.key for prop in self.related_keys]
+                if hasattr(self, "make_error"):
+                    raise self.make_error("invalid", value=value, keys=keys)
+                else:  # marshmallow 2
+                    self.fail("invalid", value=value, keys=keys)
             value = {self.related_keys[0].key: value}
         if self.transient:
             return self.related_model(**value)
@@ -127,7 +115,25 @@ class Related(fields.Field):
             ).one()
         else:
             # Use a faster path if the related key is the primary key.
-            result = query.get([value.get(prop.key) for prop in self.related_keys])
+            lookup_values = [value.get(prop.key) for prop in self.related_keys]
+            try:
+                result = query.get(lookup_values)
+            except TypeError:
+                keys = [prop.key for prop in self.related_keys]
+                if hasattr(self, "make_error"):
+                    raise self.make_error("invalid", value=value, keys=keys)
+                else:  # marshmallow 2
+                    self.fail("invalid", value=value, keys=keys)
             if result is None:
                 raise NoResultFound
         return result
+
+
+class Nested(fields.Nested):
+    """Nested field that inherits the session from its parent."""
+
+    def _deserialize(self, *args, **kwargs):
+        if hasattr(self.schema, "session"):
+            self.schema.session = self.root.session
+            self.schema.transient = self.root.transient
+        return super()._deserialize(*args, **kwargs)
